﻿using InstanceFactory.MimicAppConfig.Configuration;
using InstanceFactory.MimicAppConfig.DataEntities;
using InstanceFactory.MimicAppConfig.Xml.Serialization;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using Windows.Data.Xml.Dom;
using Windows.System;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Navigation;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238

namespace InstanceFactory.MimicAppConfig
{
  /// <summary>
  /// The app's main page.
  /// </summary>
  public sealed partial class MainPage : Page
  {
    #region Public Constructors

    /// <summary>
    /// Initializes a new instance of <see cref="MainPage"/>
    /// </summary>
    public MainPage()
    {
      this.InitializeComponent();

      ProcessConfigurationData();

      // Register handler for configuration changes.
      AppConfiguration.Current.PropertyChanged += OnConfigurationChanged;
    }

    #endregion Public Constructors


    #region Private Methods

    /// <summary>
    /// Processes the configuration data loaded from Config.Settings.xml
    /// </summary>
    private async void ProcessConfigurationData()
    {
      SetConfigurationSettingsText();

      // Load the config file
      XmlDocument xmlConfig = await ConfigurationManager.GetConfig();

      // Set the value from the appSettings section
      AppSettingsValueTextBlock.DataContext = ConfigurationManager.GetAppSettingsValue(xmlConfig, "MyConfigSetting");

      // Process the complex config data.
      ProcessComplexConfigurationData(xmlConfig);
    }

    /// <summary>
    /// Sets the text to display the configuration settings.
    /// </summary>
    private void SetConfigurationSettingsText()
    {
      GeneralInformationTextBox.DataContext 
        = String.Format("Your are running the app with {0} configuration settings.", 
            AppConfiguration.Current.UseReleaseConfig ? "RELEASE" : "DEBUG"); ;
    }

    /// <summary>
    /// Processes the complex configuration data.
    /// </summary>
    /// <param name="xmlConfig">The configuration.</param>
    private void ProcessComplexConfigurationData
      (
      XmlDocument xmlConfig
      )
    {
      XmlNodeList configDataList = xmlConfig.DocumentElement.SelectNodes("./ComplexConfigDataList/ComplexConfigData");

      if (configDataList.Count != 2)
      {
        Debug.WriteLine(String.Format("Wrong number of ComplexConfigData entries found in config. Found >{0}<, expected >2<.", configDataList.Count));
        return;
      }

      List<ComplexConfigData> complexDataList = new List<ComplexConfigData>();

      foreach (IXmlNode node in configDataList)
      {
        ComplexConfigData complexData = XmlDeserializer.Deserialize<ComplexConfigData>(node);
        Debug.WriteLine(String.Format("ComplexConfigData StringValue >{0}<, DoubleValue >{1}<, IntValue >{2}<", complexData.StringValue, complexData.DoubleValue, complexData.IntValue));
        // Save the object for later use.
        complexDataList.Add(complexData);
      }

      // Set the data context to display the values.
      ConfigDataStackPanel0.DataContext = complexDataList[0];

      ConfigDataStackPanel1.DataContext = complexDataList[1];
    }

    /// <summary>
    /// Invoked when one of the configuration properties was changed.
    /// </summary>
    /// <param name="sender">Sender of the event.</param>
    /// <param name="args">Event arguments.</param>
    private void OnConfigurationChanged
      (
      object sender,
      PropertyChangedEventArgs args
      )
    {
      // We only need to do something in case the tab selection mode has changed.
      if (!args.PropertyName.Equals("UseReleaseConfig"))
      {
        return;
      }

      ProcessConfigurationData();
    }

    /// <summary>
    /// Handles the click event of an item in the list.
    /// </summary>
    /// <param name="sender">Sender of the event.</param>
    /// <param name="args">Event-Arguments.</param>
    private void OnItemClicked
      (
      object sender,
      ItemClickEventArgs args
      )
    {
      TextBlock clickedElement = args.ClickedItem as TextBlock;

      // Only perform an action when the blog post should be opened.
      if (clickedElement == null
        || !clickedElement.Name.Equals("OpenPostTextBlock"))
      {
        return;
      }

      // We do not need to wait for the completion of the call.
      Launcher.LaunchUriAsync(new Uri("http://blog.instance-factory.com/?p=415"));
    }

    #endregion Private Methods
  }
}
